@@ -6,21 +6,21 @@ MARC::Record - Perl extension for handling MARC records
 
 =cut
 
-use 5.6.0;
+use 5.004;
 use strict;
-use warnings;
 use integer;
+use vars qw( $VERSION $ERROR $DEBUG );
 
 use MARC::Field;
 
 =head1 VERSION
 
-Version 0.06
+Version 0.10
 
 =cut
 
-our $VERSION = '0.06';
-our $ERROR;
+$VERSION = '0.10';
+$DEBUG = 0;
 
 use constant SUBFIELD_INDICATOR	=> "\x1F";
 use constant END_OF_FIELD	=> "\x1E";
@@ -123,9 +123,14 @@ sub new_from_usmarc($) {
 		or $self->_warn( "Invalid record terminator: \"$finalfield\"" );
 
 	# Walk thru the directories, and shift off the fields while we're at it
-	my $databytesused = 0;
+	# Shouldn't be any non-digits anywhere in any directory entry
 	my @directory = unpack( "A3 A4 A5" x $nfields, $dir );
+	my @bad = grep /\D/, @directory;
+	if ( @bad ) { 
+		return _gripe( "Non-numeric entries in the tag directory: ", join( ", ", map { "\"$_\"" } @bad ) );
+	}
 
+	my $databytesused = 0;
 	while ( @directory ) {
 		my $tagno = shift @directory;
 		my $len = shift @directory;
@@ -238,26 +243,26 @@ written out to a file.
 sub update_leader() {
 	my $self = shift;
 
-	my (undef,undef,$len) = $self->_build_tag_directory();
+	my (undef,undef,$reclen,$baseaddress) = $self->_build_tag_directory();
 
-	$self->_set_record_length($len);
+	$self->_set_leader_lengths( $reclen, $baseaddress );
 }
 
-=head2 _set_record_length($) 
+=head2 _set_leader_lengths($$) 
 
-Internal function for updating the leader's length.
+Internal function for updating the leader's length and base address.
 
 =cut
 
-sub _set_record_length($) {
+sub _set_leader_lengths($$) {
 	my $self = shift;
-	my $len = shift;
+	my $reclen = shift;
+	my $baseaddr = shift;
 
-	substr($self->{_leader},0,5) = sprintf("%05d",$len);
+	substr($self->{_leader},0,5)  = sprintf("%05d",$reclen);
+	substr($self->{_leader},12,5) = sprintf("%05d",$baseaddr);
 }
 
-
-
 =head2 add_fields()
 
 Adds MARC::Field objects to the end of the list.  Returns the number
@@ -265,6 +270,8 @@ of fields added, or C<undef> if there was an error.
 
 There are three ways of calling C<add_fields()> to add data to the record.
 
+=over 4
+
 =item 1 Create a MARC::Field object and add it
 
   my $author = MARC::Field->new(
@@ -272,7 +279,7 @@ There are three ways of calling C<add_fields()> to add data to the record.
 	        );
   $marc->add_fields( $author );
 
-=item 2 Add the data fields directly, and let add_fields() take care of the objectifying.
+=item 2 Add the data fields directly, and let C<add_fields()> take care of the objectifying.
 
   $marc->add_fields(
         245, "1", "0",
@@ -287,6 +294,8 @@ There are three ways of calling C<add_fields()> to add data to the record.
 	[ 650, "1", " ", a => "Raccoons." ],
 	);
 
+=back
+
 =cut
 
 sub add_fields(@) {
@@ -325,6 +334,34 @@ sub add_fields(@) {
 	return $nfields;
 }
 
+=head2 delete_field(C<$field>)
+
+Deletes a field from the record.
+
+The field must have been retrieved from the record using the 
+C<field()> method.  For example, to delete a 526 tag if it exists:
+
+    my $tag526 = $marc->field( "526" );
+    if ( $tag526 ) {
+	$marc->delete_field( $tag526 );
+    }
+
+C<delete_field()> returns the number of fields that were deleted.
+This shouldn't be 0 unless you didn't get the tag properly.
+
+=cut
+
+sub delete_field($) {
+	my $self = shift;
+	my $deleter = shift;
+	my $list = $self->{_fields};
+
+	my $old_count = @$list;
+	@$list = grep { $_ != $deleter } @$list;
+	return $old_count - @$list;
+}
+
+
 =head2 fields()
 
 Returns a list of all the fields in the record.
@@ -397,22 +434,53 @@ sub subfield($$) {
 } # subfield()
 
 
-=head2 as_string()
+=head2 as_formatted()
 
 Returns a pretty string for printing in a MARC dump.
 
 =cut
 
-sub as_string() {
+sub as_formatted() {
 	my $self = shift;
 		
 	my @lines = ( "LDR " . ($self->{_leader} || "") );
 	for my $field ( @{$self->{_fields}} ) {
-		push( @lines, $field->as_string() );
+		push( @lines, $field->as_formatted() );
 	}
 
 	return join( "\n", @lines );
-} # as_string
+} # as_formatted
+
+=head2 title()
+
+Returns the title from the 245 tag
+
+=cut
+
+sub title() {
+	my $self = shift;
+
+	my $field = $self->field(245) or return "<no 245 tag found>";
+
+	return $field->as_string;
+}
+
+=head2 author()
+
+Returns the author from the 100, 110 or 111 tag.
+
+=cut
+
+sub author() {
+	my $self = shift;
+
+	for my $tag ( qw( 100 110 111 ) ) {
+		my $field = $self->field($tag);
+		return $field->as_string() if $field;
+	}
+
+	return "<No author tag found>";
+}
 
 
 =head2 _build_tag_directory()
@@ -420,8 +488,9 @@ sub as_string() {
 Function for internal use only: Builds the tag directory that gets
 put in front of the data in a MARC record.
 
-Returns two array references, and a length: The tag directory, and the data fields themselves,
-and the length of all data, including the Leader that we expect will be added.
+Returns two array references, and two lengths: The tag directory, and the data fields themselves,
+the length of all data (including the Leader that we expect will be added),
+and the size of the Leader and tag directory.
 
 =cut
 
@@ -444,15 +513,21 @@ sub _build_tag_directory() {
 		$dataend += $len;
 	}
 
+	my $baseaddress = 
+		LEADER_LEN +    # better be 24
+		( @directory * DIRECTORY_ENTRY_LEN ) +
+				# all the directory entries
+		1;           	# end-of-field marker
+
+
 	my $total = 
-		LEADER_LEN + 				# Better be 24
-		( @directory * DIRECTORY_ENTRY_LEN ) +	# All the directory entries
-		1 +					# End-of-field marker after directory
-		$dataend + 				# Length of the fields
-		1;					# End-of-record marker
+		$baseaddress +	# stuff before first field
+		$dataend + 	# Length of the fields
+		1;		# End-of-record marker
 
 
-	return (\@fields, \@directory, $total);
+
+	return (\@fields, \@directory, $total, $baseaddress);
 }
 
 =head2 as_usmarc()
@@ -465,9 +540,8 @@ including the leader, directory and all the fields.
 sub as_usmarc() {
 	my $self = shift;
 
-	my ($fields,$directory,$len) = $self->_build_tag_directory();
-
-	$self->_set_record_length($len);
+	my ($fields,$directory,$reclen,$baseaddress) = $self->_build_tag_directory();
+	$self->_set_leader_lengths( $reclen, $baseaddress );
 
 	# Glomp it all together
 	return join("",$self->leader, @$directory, END_OF_FIELD, @$fields, END_OF_RECORD);
@@ -515,6 +589,8 @@ __END__
 
 A brief discussion of why MARC::Record is done the way it is:
 
+=over 4
+
 =item * It's built for quick prototyping
 
 One of the areas Perl excels is in allowing the programmer to 
@@ -543,12 +619,12 @@ I would suggest that if you're a cycle junkie that you use
 C<Benchmark.pm> to check to see where your bottlenecks are, and then
 decide if C<MARC::Record> is for you.
 
-
-
-
+=back
 
 =head1 SEE ALSO
 
+=over 4
+
 =item * perl4lib (L<http://www.rice.edu/perl4lib/>)
 
 A mailing list devoted to the use of Perl in libraries.
@@ -569,10 +645,17 @@ Online version of the free booklet.  An excellent overview of the MARC format.
 Follett Software Company's
 (L<http://www.fsc.follett.com/>) monthly discussion of various MARC tags.
 
+=back
 
 =head1 TODO
 
-=item * Create a meaningful test suite
+=over 4
+
+=item * Incorporate MARC.pm in the distribution.
+
+Combine MARC.pm and MARC::* into one distribution.
+
+=item * Podify MARC.pm
 
 =item * Allow regexes across the entire tag
 
@@ -595,13 +678,18 @@ Imagine something like this:
 
 =item * Modifying an existing field
 
+=back
+
 =head1 IDEAS
 
 Ideas are things that have been considered, but nobody's actually asked for.
 
-=item * Validity checking
+=over 4
 
-The leader and the 008 are the most obvious candidates for checking.
+=item * Read from MicroLIF
+
+Create a C<new_from_microlif()> function that would handle the pretty 
+MicroLIF format.  Basically, a reverse of C<as_microlif()>.
 
 =item * Create multiple output formats.
 
@@ -609,6 +697,8 @@ These could be ASCII, XML, or MarcMaker.
 
 =item * Create a clone of a record based on criteria
 
+=back
+
 =head1 LICENSE
 
 This code may be distributed under the same terms as Perl itself. 
